package Syntax::Highlight::Perl;
use Syntax::Highlight::Perl::Improved;

our $VERSION=$Syntax::Highlight::Perl::Improved::VERSION;
our @ISA=qw(Syntax::Highlight::Perl::Improved);

=head1 NAME

Syntax::Highlight::Perl - Highlighting of Perl Syntactical Structures

See L<Syntax::Highlight::Perl::Improved>

=cut

1;
